@@ -855,7 +855,7 @@ soup_cache_content_processor_wrap_input (SoupContentProcessor *processor,
 		soup_cache_entry_remove (cache, entry, TRUE);
 
 	request_time = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (msg), "request-time"));
-	response_time = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (msg), "request-time"));
+	response_time = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (msg), "response-time"));
 	entry = soup_cache_entry_new (cache, msg, request_time, response_time);
 	entry->hits = 1;
 	entry->dirty = TRUE;
@@ -928,7 +928,7 @@ soup_cache_finalize (GObject *object)
 
 	priv = SOUP_CACHE (object)->priv;
 
-	// Cannot use g_hash_table_foreach as callbacks must not modify the hash table
+	/* Cannot use g_hash_table_foreach as callbacks must not modify the hash table */
 	entries = g_hash_table_get_values (priv->cache);
 	g_list_foreach (entries, remove_cache_item, object);
 	g_list_free (entries);
@@ -1338,7 +1338,7 @@ soup_cache_clear (SoupCache *cache)
 	g_return_if_fail (SOUP_IS_CACHE (cache));
 	g_return_if_fail (cache->priv->cache);
 
-	// Cannot use g_hash_table_foreach as callbacks must not modify the hash table
+	/* Cannot use g_hash_table_foreach as callbacks must not modify the hash table */
 	entries = g_hash_table_get_values (cache->priv->cache);
 	g_list_foreach (entries, clear_cache_item, cache);
 	g_list_free (entries);
@@ -1354,7 +1354,6 @@ soup_cache_generate_conditional_request (SoupCache *cache, SoupMessage *original
 	SoupURI *uri;
 	SoupCacheEntry *entry;
 	const char *last_modified, *etag;
-	SoupMessagePrivate *origpriv;
 	GSList *f;
 
 	g_return_val_if_fail (SOUP_IS_CACHE (cache), NULL);
@@ -1382,8 +1381,7 @@ soup_cache_generate_conditional_request (SoupCache *cache, SoupMessage *original
 				      (SoupMessageHeadersForeachFunc)copy_headers,
 				      msg->request_headers);
 
-	origpriv = SOUP_MESSAGE_GET_PRIVATE (original);
-	for (f = origpriv->disabled_features; f; f = f->next)
+	for (f = soup_message_get_disabled_features (original); f; f = f->next)
 		soup_message_disable_feature (msg, (GType) GPOINTER_TO_SIZE (f->data));
 
 	if (last_modified)
